package kr.co.vividnext.sodalive.can.coupon

import kr.co.vividnext.sodalive.common.SodaException
import org.springframework.stereotype.Service

@Service
class CanCouponIssueService(private val couponNumberRepository: CanCouponNumberRepository) {
    fun validateAvailableUseCoupon(couponNumber: String, memberId: Long) {
        val canCouponNumber = checkCanCouponNumber(couponNumber)

        if (!isMultipleUse(canCouponNumber)) {
            val canCouponNumberList = couponNumberRepository.findByMemberId(memberId = memberId)
            if (canCouponNumberList.isNotEmpty()) {
                throw SodaException("해당 쿠폰은 1회만 충전이 가능합니다.")
            }
        }
    }

    private fun checkCanCouponNumber(couponNumber: String): CanCouponNumber {
        val canCouponNumber = couponNumberRepository.findByCouponNumber(couponNumber = couponNumber)
            ?: throw SodaException("잘못된 쿠폰번호입니다.\n고객센터로 문의해 주시기 바랍니다.")

        if (canCouponNumber.member != null) {
            throw SodaException("이미 사용한 쿠폰번호 입니다.")
        }

        return canCouponNumber
    }

    private fun isMultipleUse(canCouponNumber: CanCouponNumber) = canCouponNumber.canCoupon!!.isMultipleUse
}
